#include "includes.h"
#include "Console_Functions.h"

extern pPic_t Displayed_Picture;
extern EmbeddedFileSystem efs;
extern EmbeddedFile file_r, file_w;
extern pPic_t *Picture;
extern int PicIndex, PicCount;

pPic_t Load_Bitmap_to_Memory (char Name[]);
void Display_New_Picture (int New, Point X, Point Y, int Mask, int ClearFlag);


// valid command but invalid number of parameters
void Invalid_Parameters (void)
{
  WriteLn ("Invalid number of parameters!");
  return;
}

// wrong operation name
void No_Operation (void)
{
  WriteLn ("No such command!");
  return;
}

// empty line
void Empty_Line (void)
{
  // do absolutely NOTHING!
  return;
}

void Help (char Cmd[])
{
  int CommandIndex;
  CommandKey Key = _NOP;
  
  if (!My_StrCmp (Cmd, "all", NO_CASE_SENS))
    Key = _HELP;
  else
    for (CommandIndex=3; CommandIndex<NumberCommands; CommandIndex++)
      if (!My_StrCmp (Cmd, Commands[CommandIndex].Name, NO_CASE_SENS))
      {
        Key = Commands[CommandIndex].Key;
        break;
      }
  if (Key == _NOP)
  {
    WriteLn ("Wrong parameter!");
    return;
  }
  
  WriteLn ("****************************************************************************");
    
  switch (Key)
  {
  case _HELP:
    WriteLn ("1) CLS - clear screen");
    WriteLn ("2) P - draws point");
    WriteLn ("3) L - draws line");
    WriteLn ("4) T - draws triangle");
    WriteLn ("5) TF - draws triangle filled inside");
    WriteLn ("6) R - draws rectangle");
    WriteLn ("7) RF - draws rectangle filled inside");
    WriteLn ("8) E - draws ellipse");
    WriteLn ("9) C - draws circle");
    WriteLn ("10) FILES - show files of the current dir");
    WriteLn ("11) CHDIR - change current directory");
    WriteLn ("12) DISPLAY - display the named picture");
    WriteLn ("13) DISPLAY_BOX - display the named picture in a box");
    WriteLn ("14) SLIDE - slides selected pictures with some delay");
    WriteLn ("15) CLR - clear the terminal.");
    WriteLn ("16) BAUD - select the BaudRate.");
    WriteLn ("17) TOUCH_SENS - set sensitivity of the touch screen.");
    WriteLn ("--------------------------------------------------------");
    WriteLn ("For more details type: help <cmd>, where:");
    WriteLn ("<cmd> is one of the listed commands.");
    WriteLn ("Example: 'HELP TF'");
  break;
  case _CLS:
    WriteLn ("1) CLS - clear the LCD screen.");
    WriteLn ("Example: 'CLS'");
  break;
  case _P:
    WriteLn ("2) P X Y C - draw a point, where:");
    WriteLn ("X and Y are the coordinates of the point.");
    WriteLn ("C - is the color of the point");
    WriteLn ("Example: 'P 20 50 0xFF0000'");
  break;
  case _L:
    WriteLn ("3) L X1 Y1 X2 Y2 C - draw a line, where:");
    WriteLn ("X1 Y1 X2 Y2 are the coordinates of the end points of the line.");
    WriteLn ("C is the color of the line");
    WriteLn ("Example: 'L 26 50 152 18 0x00AA55'");
  break;
  case _T:
    WriteLn ("4) T X1 Y1 X2 Y2 X3 Y3 C - draw a triangle, where:");
    WriteLn ("X1 Y1 X2 Y2 X3 Y3 are the coordinates of the angles.");
    WriteLn ("C is the color of the peripheral lines.");
    WriteLn ("Example: 'T 100 50 10 160 40 140 0x00FF00'");
  break;
  case _TF:
    WriteLn ("5) TF X1 Y1 X2 Y2 X3 Y3 C CB - draw a triangle filled inside, where:");
    WriteLn ("X1 Y1 X2 Y2 X3 Y3 are the coordinates of the angles.");
    WriteLn ("C is the color of the peripheral lines.");
    WriteLn ("CB is the color of the background.");
    WriteLn ("Example: 'TF 14 52 130 26 302 126 0x78A65F 0x10F3C7'");
  break;
  case _R:
    WriteLn ("6) R X1 Y1 X2 Y2 C - draw rectangle, where:");
    WriteLn ("X1 Y1 X2 Y2 are coordinates of the opposite corners.");
    WriteLn ("C is the color of the peripheral lines.");
    WriteLn ("Example: 'R 105 170 42 25 0xFF00FF'");
  break;
  case _RF:
    WriteLn ("7) RF X1 Y1 X2 Y2 C CB - draw a rectangle filled inside, where:");
    WriteLn ("X1 Y1 X2 Y2 are coordinates of the opposite corners.");
    WriteLn ("C is the color of the peripheral lines.");
    WriteLn ("CB is the color of the background.");
    WriteLn ("Example: 'RF 105 170 42 25 0xFF00FF 0x00FF00'");
  break;
  case _E:
    WriteLn ("8) E X Y R1 R2 C - draw an ellipse, where:");
    WriteLn ("X Y are coordinates of center.");
    WriteLn ("R1/R2 is the horizontal/vertical radius.");
    WriteLn ("C is the color of the peripheral.");
    WriteLn ("Example: 'E 105 170 100 35 0x00FFFF'");
  break;
  case _C:
    WriteLn ("9) C X Y R C - draw a circle, where:");
    WriteLn ("X Y are coordinates of center.");
    WriteLn ("R is the radius.");
    WriteLn ("C is the color of the peripheral.");
    WriteLn ("Example: 'C 105 67 40 0xFF00AA'");
  break;
  case _FILES:
    WriteLn ("10) FILES - shows files in the current directory.");
    WriteLn ("!!NOTE: THE FOLDERS AND THE FILES WITHOUT    !!");
    WriteLn ("!!      EXTENSION WILL BE SHOWN THE SAME WAY !!");
    WriteLn ("Example: 'FILES'");
  break;
  case _CHDIR:
    WriteLn ("11) CHDIR Path - changes the current directory if the new exist.");
    WriteLn ("where path is the new directory or a dot(s)");
    WriteLn ("Example1: 'CHDIR Pics' - go to directory 'Pics'");
    WriteLn ("Example2: 'CHDIR .' - go back one level");
    WriteLn ("Example3: 'CHDIR ..' - go to the root directory");
  break;
  case _DISPLAY:
    WriteLn ("12) DISPLAY Picture.Ext - shows the selected picture on the display");
    WriteLn ("where picture.ext is the picture to be displayed");
    WriteLn ("NOTE1: Only *.bmp are supported");
    WriteLn ("NOTE2: If name is longer than 8 symbols it's shown like:");
    WriteLn ("abcdef~1.bmp");
    WriteLn ("Example: 'DISPLAY PIC.bmp'");
  break;
  case _DISPLAY_BOX:
    WriteLn ("13) DISPLAY_BOX Pic.Ext X1 Y1 X2 Y2- shows the selected picture");
    WriteLn ("on the display in a box with coordinates ULX, ULY and DRX, DRY.");
    WriteLn ("NOTE1: Only *.bmp are supported");
    WriteLn ("NOTE2: If name is longer than 8 symbols it's shown like:");
    WriteLn ("abcdef~1.bmp");
    WriteLn ("Example: 'DISPLAY PIC.bmp 52 106 104 232'");
  break;
  case _SLIDE:
    WriteLn ("14) SLIDE Delay Pic1.ext Pic2.ext Pic3.ext etc.");
    WriteLn ("slideshow of the selected pictures with delay");
    WriteLn ("the first parameter is the delay in ms followed");
    WriteLn ("by the names of the pictures to be slided.");
    WriteLn ("NOTE1: Only *.bmp are supported");
    WriteLn ("NOTE2: If name is longer than 8 symbols it's shown like:");
    WriteLn ("abcdef~1.bmp");
    WriteLn ("Example: 'SLIDE 2000 PIC1.bmp PIC2.bmp PIC3.bmp'");
  break;
  case _CLR:
    WriteLn ("15) CLR - clears the terminal text");
    WriteLn ("Example: 'CLR'");
  break;
  case _BAUD:
    WriteLn ("16) BAUD BaudRate - set BaudRate of the UART");
    WriteLn ("where BaudRate is one of the following:");
    WriteLn ("1 - 9600;  2 - 19200;  3 - 115200");
    WriteLn ("NOTE: It's STRONGLY RECOMMENDED to reset the terminal program");
    WriteLn ("after change of BaudRate with the new value of BaudRate.");
    WriteLn ("Example: 'BAUD 1'");
  break;
  case _TOUCH_SENS:
    WriteLn ("17) TOUCH_SENS Threshold - set the sensitivity of the display, where");
    WriteLn ("Threshold is the minimum distance (in pixels) between the last touched point");
    WriteLn ("and the current point to be triggered the event of the touch screen.");
    WriteLn ("Note: The distance is count only horizontal/vertical (no diagonal)!");
    WriteLn ("Example 'TOUCH_SENS 5'");
  break;
  default:
  break;
  }
  
  WriteLn ("****************************************************************************");
  NewLine ();
  return;
}

void Clear_Screen (void)
{
  Int32U i;
  Int32U * pData;
  LCD_UPBASE = (Int32U) LCD_Buffer[ACTIVE_BUFFER];
  ACTIVE_BUFFER = INACTIVE_BUFFER;
  pData = LCD_Buffer[ACTIVE_BUFFER];

  // clear the display memory
  for(i = 0; (C_GLCD_H_SIZE * C_GLCD_V_SIZE) > i; i++)
  {
    *pData++ = 0XFFFFFFFF;
  }
  
  // the second buffer is used to avoid the blinking when the picture is refreshed
  LCD_UPBASE = (Int32U) LCD_Buffer[ACTIVE_BUFFER];
  Int32U Size = C_GLCD_H_SIZE * C_GLCD_V_SIZE;
  for (i=0; i<Size; i++)
    LCD_Buffer[0][i] = LCD_Buffer[1][i];
  
  ACTIVE_BUFFER = INACTIVE_BUFFER;
//  BuffIndex = (BuffIndex + 1) & 1;
  LCD_UPBASE = (Int32U) LCD_Buffer[ACTIVE_BUFFER];
}

void Draw_Point (int X, int Y, long int Color)
{
  static Int32U *pointer;
  
  // check for incorrect coordinates
  if ((Y<0) || (Y>=C_GLCD_V_SIZE) || (X<0) || (X>=C_GLCD_H_SIZE))
    return;
      
//  pointer = (Int32U*) LCD_UPBASE + Y*FrameSizeX + X;
  pointer = (Int32U*) LCD_Buffer[ACTIVE_BUFFER] + Y*FrameSizeX + X;
  *pointer = Color;
  return;
}

// Using Bresenham's line algorithm:
// Algorithm description and Pseudo Code here:
// http://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm
void Draw_Line (int X1, int Y1, int X2, int Y2, long int Color)
{
	int dX, dY, sX, sY, err, e2;
	dX = abs (X2-X1);
	dY = abs (Y2-Y1);
	if (X1<X2)
		sX = 1;
	else
		sX = -1;
	if (Y1<Y2)
		sY = 1;
	else
		sY = -1;
	err = dX-dY;
	do
	{
		Draw_Point (X1, Y1, Color);
		if ((X1 == X2) && (Y1 == Y2))
			break;
		e2 = 2*err;
		if (e2 > -dY)
		{
			err = err - dY;
			X1 = X1+sX;
		}
		if (e2 < dX)
		{
			err = err + dX;
			Y1 = Y1 + sY;
		}
	}
	while (1);

	return;
}

void Draw_Triangle (int X1, int Y1, int X2, int Y2, int X3, int Y3, long int Color)
{
	Draw_Line (X1, Y1, X2, Y2, Color);
	Draw_Line (X2, Y2, X3, Y3, Color);
	Draw_Line (X3, Y3, X1, Y1, Color);
	return;
}

void Fill_Triangle (int X1, int Y1, int X2, int Y2, int X3, int Y3, long int ColorBackground)
{
  int XMin, XMax, YMin, YMax, X, Y;
  XMin = Min (X1, Min (X2, X3));
  XMax = Max (X1, Max (X2, X3));
  YMin = Min (Y1, Min (Y2, Y3));
  YMax = Max (Y1, Max (Y2, Y3));
  for (X=XMin; X<=XMax; X++)
    for (Y=YMin; Y<=YMax; Y++)
      if (Is_Inside_Point (X, Y, X1, Y1, X2, Y2, X3, Y3) == OK)
        Draw_Point (X, Y, ColorBackground);
  return;
}

void Draw_Rectangle (int X1, int Y1, int X2, int Y2, long int Color)
{
	Draw_Line (X1, Y1, X1, Y2, Color);
	Draw_Line (X1, Y1, X2, Y1, Color);
	Draw_Line (X2, Y1, X2, Y2, Color);
	Draw_Line (X1, Y2, X2, Y2, Color);
	return;
}

void Fill_Rectangle (int ULX, int ULY, int DRX, int DRY, long int Color)
{
  int X, Y;
  int MinX = Min (ULX, DRX);
  int MinY = Min (ULY, DRY);
  int MaxX = Max (ULX, DRX);
  int MaxY = Max (ULY, DRY);
  for (X=MinX; X<=MaxX; X++)
    for (Y=MinY; Y<=MaxY; Y++)
      Draw_Point (X, Y, Color);
  return;
}

// Using Bresenham's ( You're the MAN! :) ) ellipse algorithm (It's AWESOME!)
// Pascal realization here:
// http://homepage.smc.edu/kennedy_john/belipse.pdf

void Draw_Ellipse (int CX, int CY, int XRadius, int YRadius, long int Color)
{
	int X, Y, XChange, YChange, EllipseError, TwoASquare, TwoBSquare, StoppingX, StoppingY;
	TwoASquare = 2 * XRadius*XRadius;
	TwoBSquare = 2 * YRadius*YRadius;
	X = XRadius;
	Y = 0;
	XChange = YRadius*YRadius * (1-2*XRadius);
	YChange = XRadius*XRadius;
	EllipseError = 0;
	StoppingX = TwoBSquare*XRadius;
	StoppingY = 0;

	while (StoppingX >= StoppingY)	// 1st set of points, Y'> -1
	{
		Draw_4_Ellipse_Points (CX, CY, X, Y, Color);
		Y++;
		StoppingY = StoppingY + TwoASquare;
		EllipseError = EllipseError + YChange;
		YChange = YChange + TwoASquare;
		if ((2*EllipseError + XChange) > 0)
		{
			X--;
			StoppingX = StoppingX - TwoBSquare;
			EllipseError = EllipseError + XChange;
			XChange = XChange + TwoBSquare;
		}
	}

	// 1st point set is done; start the 2nd set of points

	X = 0;
	Y = YRadius;
	XChange = YRadius*YRadius;
	YChange = XRadius*XRadius * (1-2*YRadius);
	EllipseError = 0;
	StoppingX = 0;
	StoppingY = TwoASquare * YRadius;

	while (StoppingX <= StoppingY)	// 2nd set of points, Y'< -1
	{
		Draw_4_Ellipse_Points (CX, CY, X, Y, Color);
		X++;
		StoppingX = StoppingX + TwoBSquare;
		EllipseError = EllipseError + XChange;
		XChange = XChange + TwoBSquare;
		if ((2*EllipseError + YChange) > 0)
		{
			Y--;
			StoppingY = StoppingY - TwoASquare;
			EllipseError = EllipseError + YChange;
			YChange = YChange + TwoASquare;
		}	
	}

	return;
}

// A function used to draw an ellipse
void Draw_4_Ellipse_Points (int CX, int CY, int X, int Y, long int Color)
{
	Draw_Point (CX+X, CY+Y, Color);
	Draw_Point (CX-X, CY+Y, Color);
	Draw_Point (CX-X, CY-Y, Color);
	Draw_Point (CX+X, CY-Y, Color);
	return;
}

void Draw_Circle (int X, int Y, int r, long int Color)
{
	Draw_Ellipse (X, Y, r, r, Color);
	return;
}

void Files (void)
{
  extern EmbeddedFileSystem efs;
  DirList MyList;
  MyFile FileInfo;
  int Count=0;
  
  Write("Current directory: root");
  WriteLn (GlobalPath);
  if (ls_openDir (&MyList, &efs.myFs, GlobalPath))
  {
    WriteLn ("Error while opening the folder");
    return;
  }
  
  // if the current folder contains more files/folders --> get the next one
  while (ls_getNext(&MyList)==0)
  {
    // forming the name of the file/folder
    // the structure of the FileInfo consist of:
    // Name (up to 8 characters), Extension (none for folders) and 
    // FullName (both of them combined and separated by '.')
    Form_File_Name (&FileInfo, MyList.currentEntry.FileName);
    WriteLn (FileInfo.FullName);
    Count++;
  }
  if (!Count)
    WriteLn ("Folder is empty!");
  return;
}

void Change_Directory (char Path[])
{
  DirList MyList;
  int i, PrevSlash=0, LastSlash=0;
  if (My_StrCmp (Path, ".", NO_CASE_SENS)==0)    // go to previous directory
  {
    for (i=1; GlobalPath[i]; i++)
      if (GlobalPath[i] == '/')
      {
        PrevSlash = LastSlash;
        LastSlash = i;
      }
    GlobalPath[PrevSlash + 1] = 0;
    return;
  }
  
  if (My_StrCmp (Path, "..", NO_CASE_SENS)==0)   // go to root dir
  {
    GlobalPath[1] = 0;
    return;
  }
  
  // go to selected folder
  sprintf (GlobalPath, "%s%s/", GlobalPath, Path);
  
  if (ls_openDir (&MyList, &efs.myFs, GlobalPath))  // check if it doesn't exist
  {
    WriteLn ("Inexisting folder!");
    Change_Directory ("."); // go back to the prevoius dir
    return;
  }
  return;
}

void Display (char Name[])
{
  Point UL = {0, 0}, DR = {C_GLCD_H_SIZE-1, C_GLCD_V_SIZE-1};
  if (Displayed_Picture)
    free (Displayed_Picture);
  
  Displayed_Picture = Load_Bitmap_to_Memory (Name);
  
  if (Displayed_Picture == NULL)
    return;
  
  GLCD_ShowPic_in_Box (Displayed_Picture, UL, DR, 0x000000, FALSE);
  return;
}

void Display_Box (char Name[], Point UL, Point DR, int ClearFlag)
{
  if (Displayed_Picture)
    free (Displayed_Picture);
  
  Displayed_Picture = Load_Bitmap_to_Memory (Name);
  
  if (Displayed_Picture == NULL)
    return;
  
  GLCD_ShowPic_in_Box (Displayed_Picture, UL, DR, 0x000000, ClearFlag);
  return;
}

void Slide (unsigned long int Delay, int argc, char **argv)
{
  Point UL = {0, 0}, DR = {C_GLCD_H_SIZE-1, C_GLCD_V_SIZE-1};
  int i;
  char Dummy[2];
  char FullPathName[MAX_PATH_LENGTH + MAX_FILE_NAME_LENGTH + MAX_FILE_EXT_LENGTH];
  PicCount = 0;
  for (i=2; i<argc; i++)
  {
    sprintf (FullPathName, "%s%s", GlobalPath, argv[i]);
    if (file_fopen (&file_r, &efs.myFs, FullPathName, 'r') == 0)
    {
      PicCount++;
      file_fclose (&file_r);
    }
  }
  if (PicCount == 0)
    return;
  
  WriteLn ("Loading pictures into memory...");
  Picture = (pPic_t*) malloc (PicCount*sizeof(Pic_t));
  PicCount = 0;
  for (i=2; i<argc; i++)
  {
    sprintf (FullPathName, "%s%s", GlobalPath, argv[i]);
    if (file_fopen (&file_r, &efs.myFs, FullPathName, 'r') == 0)
      Picture[PicCount++] = Load_Bitmap_to_Memory (argv[i]);
  }
  Write ("Pictures loaded! Press Enter to interrupt sliding!");
  do
  {
    Display_New_Picture (NEXT, UL, DR, 0x000000, TRUE);
    DelayResolution100us (10*Delay);
  }
  while (!ReadLn (Dummy));
  
  for (i=0; i<PicCount; i++)
    free (Picture[i]);
  free (Picture);
}

void Clear_Terminal (void)
{
  Write("\f");
}